import sys
import pygame
import pytmx
from pytmx.util_pygame import load_pygame
import const
import util


# Data expected from .tmx file:
# - Layers are: Floor, Solid, Inner Walls
# - Object groups are: Slots, Operators, Doors, Breaches, Start, Track, Wires
# - Operators should have property 'optype' in 'not', 'and', 'or', or 'xor'
# - Inner walls should have property 'iwallcol' among directions (tblr)
# - Tracks should have property 'in' and 'out' among directions (tblr), indicating direction in and out
# - Tracks should have off and on states as animation
# - Tracks should not loop or overlap (otherwise behavior is undefined)
# - Breaches should have property 'dir' indicating exit direction
# - Breaches should have on and off states as animation
# - Tracks should start at a breach at the correct direction and end at the border of the level
# - Doors should have closed and open states as animation
# - Wires should always have only a single end at a Source and possibly multiple door ends; they cannot end at the border of a level
# - Wires should have on and off states as animation
# - Wires should have property 'dirs' indicating all directions
# - Sources should have property 'dir' indicating exit direction
# - Start should have a single object in the start position

class LevelManager(object):

    IW_WIDTH = 5
    IW_BBS = {
        't':  (0,0,32,IW_WIDTH),
        'b':  (0,32-IW_WIDTH,32,IW_WIDTH),
        'l':  (0,0,IW_WIDTH,32),
        'r':  (32-IW_WIDTH,0,IW_WIDTH,32),
        'tl': (0,0,IW_WIDTH,IW_WIDTH),
        'tr': (32-IW_WIDTH,0,IW_WIDTH,IW_WIDTH),
        'bl': (0,32-IW_WIDTH,IW_WIDTH,IW_WIDTH),
        'br': (32-IW_WIDTH,32-IW_WIDTH,IW_WIDTH,IW_WIDTH)
        }

    def __init__(self, filename):
        tm = load_pygame(filename)
        self.pixel_size = tm.width * tm.tilewidth, tm.height * tm.tileheight
        self.tmx_data = tm
        self.create_solids()

    def create_solids(self):
        self.solids = [[False for j in xrange(self.tmx_data.height)] for i in xrange(self.tmx_data.width)]
        if "Solid" not in self.tmx_data.layernames:
            print "Error loading level: Solid layer not found"
        layer = self.tmx_data.layernames["Solid"]
        for x, y, image in layer.tiles():
            self.solids[x][y] = True

    def get_inner_wall_rects(self):
        if "Inner Walls" not in self.tmx_data.layernames:
            print "Error loading level: Inner Walls layer not found"
        layer = self.tmx_data.layernames["Inner Walls"]
        rects = []
        for tx, ty, gid in layer.iter_data():
            if gid == 0: continue
            properties = self.tmx_data.get_tile_properties_by_gid(gid)
            if properties == None: continue
            for rect_type in properties["iwallcol"].split():
                r = self.IW_BBS[rect_type]
                x, y = util.get_xy_at_tile(tx, ty)
                rects.append(pygame.Rect(x+r[0], y+r[1], r[2], r[3]))
        return rects

    def render_map(self, surface):
        if self.tmx_data.background_color:
            surface.fill(pygame.Color(self.tmx_data.background_color))
        for layer in self.tmx_data.visible_layers:
            if isinstance(layer, pytmx.TiledTileLayer):
                self.render_tile_layer(surface, layer)

    def render_tile_layer(self, surface, layer):
        tw = self.tmx_data.tilewidth
        th = self.tmx_data.tileheight
        surface_blit = surface.blit
        for x, y, image in layer.tiles():
            surface_blit(image, (x * tw, y * th))

    def get_layer(self, name):
        return self.tmx_data.layernames[name]

    def get_start_pos(self):
        try:
            tmx_start = self.tmx_data.layernames["Start"][0]
        except:
            print "Error loading level: Start position not found"
            sys.exit()
        return (tmx_start.x, tmx_start.y)

    def render_image_layer(self, surface, layer):
        if layer.image:
            surface.blit(layer.image, (0, 0))

    def is_solid(self, x, y):
        if x < 0 or y < 0 or x >= self.tmx_data.width or y >= self.tmx_data.height: return True
        return self.solids[x][y]

